import ArrowUpwardIcon from '@mui/icons-material/ArrowUpward'
import NotificationsActiveIcon from '@mui/icons-material/NotificationsActive'
import {
  Accordion,
  AccordionDetails,
  AccordionSummary,
  Button,
  Paper,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableHead,
  TableRow,
  Typography,
} from '@mui/material'
import { styled } from '@mui/system'
import { useState } from 'react'

import Loading from 'components/loading'
import Page from 'components/page'
import { differenceInDays, format, isBefore } from 'date-fns'
import { Guest, Role } from 'interfaces'
import { Trans, useTranslation } from 'react-i18next'
import { Link } from 'react-router-dom'
import { getRoleName, getRoleOuName } from 'utils'
import SponsorGuestButtons from '../../components/sponsorGuestButtons'

interface GuestProps {
  persons: Guest[]
}

interface PersonLineProps {
  person: Guest
  role: Role
}

interface StatusProps {
  person: Guest
  role: Role
}

interface GuestTableProps {
  guests: Guest[]
  emptyText: string
  marginWidth: number
}

interface FrontPageProps {
  guests: Guest[]
  loading: boolean
}

const StyledTableRow = styled(TableRow)({
  borderTop: '0',
  borderLeft: '0',
  borderRight: '0',
  borderBottom: '2px solid',
  borderColor: 'black',
  borderRadius: '0',
})

const StyledAccordion = styled(Accordion)({
  borderStyle: 'none',
  boxShadow: 'none',
  margin: 'auto',
})

const StyledAccordionSummary = styled(AccordionSummary)({
  borderStyle: 'solid',
  borderColor: 'black',
  borderWidth: '0.125rem',
  borderRadius: '1%',
})

const StyledTableHeadCell = styled(TableCell)({
  fontWeight: 'bold',
})

const StyledTableHead = styled(TableHead)(({ theme }) => ({
  borderTop: '0',
  borderLeft: '0',
  borderRight: '0',
  borderBottom: '0.1875rem solid',
  borderColor: theme.palette.secondary.main,
  borderRadius: '0',
}))

const calculateStatus = (person: Guest, role: Role): [string, number] => {
  const today = new Date()
  today.setHours(0, 0, 0, 0)
  let status = ''
  const days = differenceInDays(role.end_date, today)

  if (!person.registered) {
    status = 'waitingForGuest'
    if (person.invitation_status === 'invalidEmail') {
      status = 'invalidEmail'
    } else if (person.invitation_status !== 'active') {
      status = 'invitationExpired'
    }
  } else if (person.registered && !person.verified) {
    status = 'waitingForSponsor'
  } else if (person.registered && person.verified) {
    if (isBefore(role.end_date, today)) {
      status = 'expired'
    } else if (days <= 7) {
      status = 'expiring'
    } else {
      status = 'active'
    }
  }
  return [status, days]
}

const Status = ({ person, role }: StatusProps) => {
  const { t } = useTranslation('common')
  const [status, days] = calculateStatus(person, role)

  switch (status) {
    case 'active':
      return (
        <TableCell sx={{ color: 'success.main' }} align="left">
          {t('statusText.active')}
        </TableCell>
      )
    case 'expired':
      return (
        <TableCell sx={{ color: 'error.main' }} align="left">
          {t('statusText.expired')}
        </TableCell>
      )
    case 'waitingForGuest':
      return (
        <TableCell sx={{ color: 'blue' }} align="left">
          {t('statusText.waitingForGuest')}
        </TableCell>
      )
    case 'waitingForSponsor':
      return (
        <TableCell sx={{ color: 'blue' }} align="left">
          {t('statusText.waitingForSponsor')}
        </TableCell>
      )
    case 'invitationExpired':
      return (
        <TableCell sx={{ color: 'error.main' }} align="left">
          {t('statusText.invitationExpired')}
        </TableCell>
      )
    case 'expiring':
      return (
        <TableCell sx={{ color: 'blue' }} align="left">
          <Trans t={t} i18nKey="statusText.expiring" count={days} />
        </TableCell>
      )
    case 'invalidEmail':
      return (
        <TableCell sx={{ color: 'error.main' }} align="left">
          <Trans t={t} i18nKey="statusText.invalidEmail" count={days} />
        </TableCell>
      )
    default:
      return (
        <TableCell sx={{ color: 'error.main' }} align="left">
          {t('statusText.waiting')}
        </TableCell>
      )
  }
}

const sortByDate = (
  guestRole: { guest: Guest; role: Role }[]
): { guest: Guest; role: Role }[] =>
  guestRole.sort(
    (a, b) => a.role.end_date.getTime() - b.role.end_date.getTime()
  )

const sortByStatus = (guests: Guest[]): { guest: Guest; role: Role }[] => {
  let activeRoleAndPerson: { guest: Guest; role: Role }[] = []
  let expiringRoleAndPerson: { guest: Guest; role: Role }[] = []
  let expiredRoleAndPerson: { guest: Guest; role: Role }[] = []
  let status

  guests.forEach((guest) =>
    guest.roles.forEach((role) => {
      ;[status] = calculateStatus(guest, role)
      if (status === 'active') {
        activeRoleAndPerson.push({ guest, role })
      } else if (status === 'expiring') {
        expiringRoleAndPerson.push({ guest, role })
      } else {
        expiredRoleAndPerson.push({ guest, role })
      }
    })
  )

  activeRoleAndPerson = sortByDate(activeRoleAndPerson)
  expiringRoleAndPerson = sortByDate(expiringRoleAndPerson)
  expiredRoleAndPerson = sortByDate(expiredRoleAndPerson)

  return expiringRoleAndPerson
    .concat(activeRoleAndPerson)
    .concat(expiredRoleAndPerson)
}

const PersonLine = ({ person, role }: PersonLineProps) => {
  const [t] = useTranslation(['common'])

  return (
    <StyledTableRow>
      <TableCell component="th" scope="row">
        {`${person.first} ${person.last}`}
      </TableCell>
      <TableCell align="left">{getRoleName(role)}</TableCell>
      <Status person={person} role={role} />
      <TableCell align="left"> {format(role.end_date, 'yyyy-MM-dd')}</TableCell>
      <TableCell align="left">{getRoleOuName(role)}</TableCell>
      <TableCell align="left">
        <Button
          variant="contained"
          color="secondary"
          component={Link}
          to={`/sponsor/guest/${person.pid}`}
        >
          {t('common:details')}
        </Button>
      </TableCell>
    </StyledTableRow>
  )
}

const GuestTable = ({ guests, emptyText, marginWidth }: GuestTableProps) => {
  const { t } = useTranslation('common')

  return (
    <TableContainer
      component={Paper}
      sx={{
        boxShadow: 'none',
        borderRadius: '0rem',
        minWidth: marginWidth,
      }}
    >
      <Table sx={{ minWidth: marginWidth }} aria-label="simple table">
        <StyledTableHead>
          <TableRow>
            <StyledTableHeadCell>{t('common:name')}</StyledTableHeadCell>
            <StyledTableHeadCell>{t('common:role')}</StyledTableHeadCell>
            <StyledTableHeadCell>{t('common:status')}</StyledTableHeadCell>
            <StyledTableHeadCell>{t('common:endDate')}</StyledTableHeadCell>
            <StyledTableHeadCell>{t('common:department')}</StyledTableHeadCell>
            <StyledTableHeadCell />
          </TableRow>
        </StyledTableHead>
        <TableBody>
          {guests.length > 0 ? (
            sortByStatus(guests).map((personRole) => (
              <PersonLine
                key={`${personRole.guest.first} ${personRole.guest.last} ${personRole.role.id}`}
                role={personRole.role}
                person={personRole.guest}
              />
            ))
          ) : (
            <StyledTableRow>
              <TableCell> {emptyText}</TableCell>
            </StyledTableRow>
          )}
        </TableBody>
      </Table>
    </TableContainer>
  )
}

const InvitedGuests = ({ persons }: GuestProps) => {
  const [activeExpanded, setActiveExpanded] = useState(false)

  // Show guests that have not responded to the invite yet
  let guests = persons.length > 0 ? persons : []

  if (guests.length > 0) {
    guests = guests.filter((person) => !person.registered)
  }
  const [t] = useTranslation(['common'])
  return (
    <StyledAccordion
      expanded={activeExpanded}
      onChange={() => {
        setActiveExpanded(!activeExpanded)
      }}
    >
      <StyledAccordionSummary
        expandIcon={<ArrowUpwardIcon sx={{ fill: 'black' }} />}
      >
        <Typography variant="h2">
          {`${t('common:sentInvitations')} (${guests.length})`}
        </Typography>
      </StyledAccordionSummary>
      <AccordionDetails>
        <Typography
          variant="body1"
          sx={{ marginTop: '1rem', marginBottom: '1rem' }}
        >
          {t('common:sentInvitationsDescription')}
        </Typography>
        <GuestTable
          guests={guests}
          emptyText={t('common:noInvitations')}
          marginWidth={650}
        />
      </AccordionDetails>
    </StyledAccordion>
  )
}

const ActiveGuests = ({ persons }: GuestProps) => {
  const [activeExpanded, setActiveExpanded] = useState(false)

  // Show all verified guests
  let guests = persons.length > 0 ? persons : []
  if (guests.length > 0) {
    guests = guests.filter((person) => person.verified)
  }
  const [t] = useTranslation(['common'])
  return (
    <StyledAccordion
      expanded={activeExpanded}
      onChange={() => {
        setActiveExpanded(!activeExpanded)
      }}
    >
      <StyledAccordionSummary
        expandIcon={<ArrowUpwardIcon sx={{ fill: 'black' }} />}
      >
        <Typography variant="h2">
          {`${t('common:activeGuests')} (${guests.length})`}
        </Typography>
      </StyledAccordionSummary>
      <AccordionDetails>
        <Typography
          variant="body1"
          sx={{ marginTop: '1rem', marginBottom: '1rem' }}
        >
          {t('common:activeGuestsDescription')}
        </Typography>
        <GuestTable
          guests={guests}
          emptyText={t('common:noActiveGuests')}
          marginWidth={1000}
        />
      </AccordionDetails>
    </StyledAccordion>
  )
}

const WaitingGuests = ({ persons }: GuestProps) => {
  const [waitingExpanded, setWaitingExpanded] = useState(false)

  // Show guests that have completed the registration but are not verified yet
  let guests = persons.length > 0 ? persons : []
  if (guests.length > 0) {
    guests = guests.filter((person) => person.registered && !person.verified)
  }
  const [t] = useTranslation(['common'])

  return (
    <StyledAccordion
      expanded={waitingExpanded}
      onChange={() => {
        setWaitingExpanded(!waitingExpanded)
      }}
    >
      <StyledAccordionSummary
        expandIcon={<ArrowUpwardIcon sx={{ fill: 'black' }} />}
      >
        <Typography variant="h2">
          {`${t('common:waitingGuests')} (${guests.length})`}{' '}
          {guests.length > 0 && (
            <NotificationsActiveIcon
              sx={{ color: 'error.main', fontSize: '1.625rem' }}
            />
          )}
        </Typography>
      </StyledAccordionSummary>
      <AccordionDetails>
        <Typography
          variant="body1"
          sx={{ marginTop: '1rem', marginBottom: '1rem' }}
        >
          <Trans i18nKey="common:waitingGuestsDescription">
            Her godkjenner du gjester som har <b>registrert seg manuelt</b>.
            Gjester som har FEIDE-bruker trenger ikke godkjenning.
          </Trans>
        </Typography>
        <GuestTable
          guests={guests}
          emptyText={t('common:noWaitingGuests')}
          marginWidth={650}
        />
      </AccordionDetails>
    </StyledAccordion>
  )
}

function FrontPage({ guests, loading }: FrontPageProps) {
  return (
    <Page pageWidth>
      <SponsorGuestButtons yourGuestsActive />

      {loading ? (
        <Loading />
      ) : (
        <>
          <InvitedGuests persons={guests} />
          <br />
          <WaitingGuests persons={guests} />
          <br />
          <ActiveGuests persons={guests} />
        </>
      )}
    </Page>
  )
}

export default FrontPage
